using namespace JS;
using namespace js;

extern JS::PersistentRootedObject global;
extern JSContext* cx;
extern JSCompartment* oldCompartment;


int testStructuredCloneReaderInit(int *argc, char ***argv) {
  jsfuzz_initg(&cx, &global);
  //JS_SetGCZeal(cx, 2, 1);
  return 0;
}

int testStructuredCloneReaderFuzz(const uint8_t* buf, size_t size)
{
    {
//        RootedObject global(cx, unrootedGlobal);
        if (!size) goto end;

        const size_t kSegmentAlignment = 8;

        size_t size_diff = kSegmentAlignment - (size % kSegmentAlignment);
        size_t buf_size = size + size_diff;

        auto clonebuf = js::MakeUnique<JSStructuredCloneData>(0, 0, buf_size);
        if (!clonebuf || !clonebuf->Init(buf_size, buf_size)) {
            ReportOutOfMemory(cx);
            goto end;
        }

        memset(clonebuf->Start(), '\0', buf_size);
        js_memcpy(clonebuf->Start(), buf, size);

        JS::StructuredCloneScope scope = JS::StructuredCloneScope::DifferentProcess;

        RootedValue deserialized(cx);
        if (!JS_ReadStructuredClone(cx, *clonebuf,
                                JS_STRUCTURED_CLONE_VERSION,
                                scope,
                                &deserialized, nullptr, nullptr))
        {
            goto free;
        }

        ValueToSource(cx, deserialized);

        free:
            // Do I have to free anything here?
            goto end;
    }

end:

    JS::PrepareForFullGC(cx);
    JS::GCForReason(cx, GC_NORMAL, JS::gcreason::API);
    return 0;
}

MOZ_LIBFUZZER_INTERFACE_RAW(testStructuredCloneReaderInit, testStructuredCloneReaderFuzz, StructuredCloneReader);
